theme = [
    ('list-header', '', '', '', '', ''),
    ('list-header-column', 'black', 'light gray', '', 'black', 'light gray'),
    ('list-header-column-separator', 'black', 'light gray', '', 'black', 'light gray'),
    ('striped-table-row', 'white', 'dark gray', '', 'white', 'g27'),
    ('reveal focus', 'black', 'dark cyan', 'standout', 'black', 'dark cyan'),
    ('message status', 'white', 'dark blue', 'standout', 'white', 'dark blue'),
    ('message error', 'white', 'dark red', 'standout', 'white', 'dark red'),
    ('status', 'dark magenta', 'black', '', 'dark magenta', 'black'),
    ('flash off', 'black', 'black', 'standout', 'black', 'black'),
    ('flash on', 'white', 'black', 'standout', 'white', 'black'),
    ('pop_up', 'white', 'black', '', 'white', 'black'),
    ('button action', 'white', 'light red', '', 'white', 'light red'),
    ('button cancel', 'black', 'light gray', '', 'black', 'light gray'),
]
